/******************************************************************************

                               Copyright (c) 2010
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <sys/mman.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <sys/ioctl.h>
#include "pmcu.h"

/**
   \defgroup LQ_PM_PRG Program Guide
   \ingroup LQ_PM
   <center>
   <B>"Power Management Utility ( PMCU_Utility )" </B>.
   </center>    
   \section Purpose Purpose
    The PMCU_Utility is a command line utility that allows interaction with the PMCU kernel driver.
    The utility could be started as a application on user space level and use directly the PMCU Ioctl's.
   \n
   \section Usage Usage
   The PMCU_Utility may called without any parameters. In this case a help menue is displayed to 
   list all of the available command line parameters.\n
   <B>Example:</B> :_><I>/opt/lantiq/bin/pmcu_utility</I>\n
   
   \code
      # /opt/lantiq/bin/pmcu_utility
      pmcu:please specify parameters
      pmcu_utility version = 2.0.4.1
      Usage:/opt/lantiq/bin/pmcu_utility [options] [parameter] ... options:
      -h --help            Display help information
      -s --set=STATE       Set module state
      -m --module=MODULE   Specify module
      -n --sub=SUBNUM      Specify submodule number
      -g --get             Get module status
      -r --req             enable/disable the acception of a powerstate request (0:disable,1:enable)
      -l --log             set log level for PMCU kernel driver (0:quiet,1:ERROR,2:+INFO,3:+DEBUG)
      -f --feature         power feature which should be enabled/disabled; 
      -e --event           Get module event
      -v --version         Display version information of PMCU
      -d --psinfo          Display powerState and Registry info of all modules
      MODULE=cpu/eth/usb/dsl/wlan/dect/fxs/fxo/ve/ppe/switch/uart/spi/sdio/pci/vlynq/deu/cpu_ps/gptc/usif_uart/usif_spi/pcie
      STATE=0/1/2/3 corresponds with D0/D1/D2/D3
      Example: /opt/lantiq/bin/pmcu_utility -s 3 -m cpu -n 0
      Example: /opt/lantiq/bin/pmcu_utility -m wlan -f off
      Example: /opt/lantiq/bin/pmcu_utility -r 1
   \endcode
   
   If you want to initiate a power state change for a module call first the pmcu_utility with the option -d to
   get a list of all registered modules and their current power state.\n
   
   \code
      # /opt/lantiq/bin/pmcu_utility -d
      Legend:
              r       :registered (-=not registered, 1=unregistered, 2=registered)
              Sub#    :Submodule or module instance
    		  ps      :power state (-1=invalid state, D0=OnState, D1=lp1, D2=lp2, D3=LowPower n.d.=not defined)

      Modules         Sub0    Sub1    Sub2    Sub3    Sub4    Sub5
                      r/ps    r/ps    r/ps    r/ps    r/ps    r/ps
      PMCU            -       -       -       -       -       -
      CPU             2/D0    -       -       -       -       -
      ETH             -       -       -       -       -       -
      USB             -       -       -       -       -       -
      DSL             -       -       -       -       -       -
      WLAN            -       -       -       -       -       -
      DECT            -       -       -       -       -       -
      FXS             -       -       -       -       -       -
      FXO             -       -       -       -       -       -
      VE              -       -       -       -       -       -
      PPE             2/D0    -       -       -       -       -
      SWITCH          -       -       -       -       -       -
      UART            -       -       -       -       -       -
      SPI             2/D0    -       -       -       -       -
      SDIO            -       -       -       -       -       -
      PCI             2/D0    -       -       -       -       -
      VYLINQ          -       -       -       -       -       -
      DEU             -       -       -       -       -       -
      CPU_PS          -       -       -       -       -       -
      GPTC            -       -       -       -       -       -
      USIF_UART       -       -       -       -       -       -
      USIF_SPI        -       -       -       -       -       -
      PCIE            2/D0    -       -       -       -       -
   \endcode

   The acception for the powerstate request must be enabled once because it is disabled by default.\n 
   <B>Example:</B> :_><I>/opt/lantiq/bin/pmcu_utility -r 1</I>\n 
   The following example requests the power state "3" for module "cpu" with submodule number "0"\n
   <B>Example:</B> :_><I>/opt/lantiq/bin/pmcu_utility -s 3 -m cpu -n 0</I>\n
   \n
   \n
*/


/**
   \defgroup LQ_PM Power Management Utility
   \ingroup LQ_COC
*/
/*@{*/


/* PMCU_UTILITY version  */
#define COC_PMCU_UTILITY_VERSION  "2.0.5.0"


static int c_opt_g;                 /**< parameter(option) fetch from command list */
static int input_flag_g = 0;        /**< help flag to indicate the first time entering the endless loop */
static void (*func)() = NULL;       /**< function pointer to the currently called function */
static IFX_PMCU_MODULE_t module_g;  /**< hold the current module id */
static char sub_g = 0;              /**< hold the current subModule number */
static IFX_PMCU_STATE_t state_g;    /**< hold the current module state (D0,D1,D2,D3) */
static int log_level_g = 1;         /**< Loglevel control for the module: 0=quiet, 1=only ERRORS, 2=+INFO, 3=+DEBUG */
static int req_ctrl_g;              /**< powerstate request control for the module: 0=disabled, 1=enabled */
static int pwrFea_g;                /**< power feature enable/disable: OFF=disable, ON=enable */

/** Power state translation */
static char* pmcu_ps[] = {
   "INVALID / DISABLED" ,
   "D0" ,
   "D1" ,
   "D2" ,
   "D3" ,
};


/* module names; array correspond to enum IFX_PMCU_MODULE_t */
static char* ifx_pmcu_mod[IFX_PMCU_MODULE_ID_MAX+1] = {
   "PMCU" ,
   "CPU " ,
   "ETH " ,
   "USB " ,
   "DSL " ,
   "WLAN" ,
   "DECT" ,
   "FXS " ,
   "FXO " ,
   "VE  " ,
   "PPE " ,
   "SWITCH" ,
   "UART" ,
   "SPI " ,
   "SDIO" ,
   "PCI " ,
   "VYLINQ" ,
   "DEU " ,
   "CPU_PS" ,
   "GPTC" ,
   "USIF_UART" ,
   "USIF_SPI" ,
   "PCIE" ,
   "INVALID" ,
};


#if 0
/* these two variables are used to fetch the current major number of the pmcu kernel driver to create a device nod */
FILE* fPtr = NULL;
char major[5];
#endif


/**
   Display the device list from the PMCU kernel driver.\n
   Shows an overview about the registered modules and their current PowerState

   \return None
*/
static void display_pmcu_info(void)
{
   system("cat /proc/pmcu/device_list");
   return;
}

/**
   Display the current version id of the PMCU kernel driver.\n

   \return None
*/
static void display_version(void)
{
   system("cat /proc/pmcu/version");
   return;
}

/**
   pmcu_utility help menu.\n
   Shows an overview about the parameters and the options which are
   supported by the pmcu_utility

   \return None
*/
static void pmcu_help(void)
{
   printf("pmcu_utility version = %s\n",COC_PMCU_UTILITY_VERSION);
   printf("Usage:/opt/lantiq/bin/pmcu_utility [options] [parameter] ...\n");
   printf("options:\n");
   printf("-h --help            Display help information\n");
   printf("-s --set=STATE       Set module state\n");
   printf("-m --module=MODULE   Specify module\n");
   printf("-n --sub=SUBNUM      Specify submodule number\n");
   printf("-g --get             Get module status\n");
   printf("-l --log             set log level for PMCU kernel driver (0:quiet,1:ERROR,2:+INFO,3:+DEBUG)\n");
   printf("-r --req             enable/disable the acception of a powerstate request (0:disable,1:enable)\n");
   printf("-f --feature         power feature which should be enabled/disabled (ON, OFF)\n");
   printf("-e --event           Get module event\n");
   printf("-v --version         Display version information of PMCU\n");
   printf("-d --psinfo          Display powerState and Registry info of all modules\n");
   printf("MODULE=cpu/eth/usb/dsl/wlan/dect/fxs/fxo/ve/ppe/switch/uart/spi/sdio/pci/vlynq/deu/cpu_ps/gptc/usif_uart/usif_spi/pcie\n");
   printf("STATE=0/1/2/3 corresponds with D0/D1/D2/D3\n");
   printf("Example: /opt/lantiq/bin/pmcu_utility -s 3 -m cpu -n 0\n");
   printf("Example: /opt/lantiq/bin/pmcu_utility -m wlan -f off\n");
   printf("Example: /opt/lantiq/bin/pmcu_utility -r 1\n");
   return;
}


/**
   Enable/Disable the existing power feature via the PMCU ioctl-interface 
   for the given module/subModule. 

   The input for state, module and subModule are given via global variables pwrFea_g, module_g, sub_g.
   
   \return None
*/
static void set_pwrFeaState(void)
{
   int ret, fd;
   IFX_PMCU_MODULE_PWR_FEATURE_STATE_t param={0,0,0};

   fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd<0) {
      perror("cannot open device!\n");
      exit(-1);
   }
   param.pmcuModule=module_g;
   param.pmcuModuleNr=sub_g;
   param.pmcuPwrFeatureState=pwrFea_g;
   ret=ioctl(fd,IFX_PMCU_PWR_FEATURE_SWITCH,&param);
   if(ret<0){
      perror("ioctl IFX_PMCU_PWR_FEATURE_SWITCH failed!\n");
   }
   close(fd);
   return;
}


/** Control the acceptance of the powerstate request for the
    PMCU module. To have a central possibility for a global
    enable/disable of the powerstate requests coming from other
    modules, this IOCTL is introduced. The input is given via a
    global variable req_ctrl_g.
           - 0 =   All powerstate requests received by the PMCU
                   will be rejected. Default
           - 1  =  Powerstate requests are accepted by the PMCU

   \return None
*/
static void set_enablePwrStateRequest(void)
{
   int ret, fd;

   fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd<0) {
      perror("cannot open device!\n");
      exit(-1);
   }
   ret=ioctl(fd,IFX_PMCU_REQ_CTRL,&req_ctrl_g);
   if(ret<0){
      perror("ioctl IFX_PMCU_REQ_CTRL failed!\n");
   }
   close(fd);
   return;
}



/** Change the log level for the PMCU module
   The input is given via a global variable log_level_g which specifies the log level
          -  0 = quiet
          -  1 = only ERRORS (default)
          -  2 = + INFO
          -  3 = + DEBUG

   \return None
*/
static void set_logLevel(void)
{
   int ret, fd;

   fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd<0) {
      perror("cannot open device!\n");
      exit(-1);
   }
   ret=ioctl(fd,IFX_PMCU_LOG_LEVEL,&log_level_g);
   if(ret<0){
      perror("ioctl IFX_PMCU_LOG_LEVEL failed!\n");
   }
   close(fd);
   return;
}


/**
   The set_state function request a new power state via the PMCU
   ioctl-interface for the given module/subModule.

   The input for state, module and subModule are given via global variables state, module, sub.
   
   \return None
*/
static void set_state(void)
{
   int ret, fd;
   IFX_PMCU_MODULE_STATE_t param={0,0,0};

   fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd<0) {
      perror("cannot open device!\n");
      exit(-1);
   }
   param.pmcuModule=module_g;
   param.pmcuModuleNr=sub_g;
   param.pmcuState=state_g;
   ret=ioctl(fd,IFX_PMCU_STATE_REQ,&param);
   if(ret<0){
      perror("ioctl IFX_PMCU_STATE_REQ failed!\n");
   }
   close(fd);
   return;
}


/**
   The get_state function read the current power state from the 
   given module and return it.

   \return  current power state of the module given via module_g variable.
            Returned values -> IFX_PMCU_STATE_t
*/
static void get_state(void)
{
   IFX_PMCU_MODULE_STATE_t param={0,0,IFX_PMCU_STATE_INVALID};
   int ret, fd;

   fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd<0) {
      perror("cannot open device!\n");
      exit(-1);
   }
   param.pmcuModule=module_g;
   param.pmcuModuleNr=sub_g;
   ret=ioctl(fd,IFX_PMCU_STATE_GET,&param);
   if(ret<0){
      perror("module not registered, therefore no PowerState available\n");
   }else{
      printf("Current PowerState for module %s is %s\n", ifx_pmcu_mod[param.pmcuModule], pmcu_ps[param.pmcuState]);
   }
   close(fd);
   return;
}

/**
   Function not supported anymore.

   \return  None
*/
static void get_info(void)
{
#if 0
   IFX_PMCU_INFO_t param={0};
   IFX_int32_t ret;
   int fd=open("/dev/ifx_pmcu",O_RDWR);
   if(fd==-1)
   {
      printf("cannot open device!\n");
      exit(-1);
   }
   param.pmcuModule=module_g;
   param.pmcuModuleNr=sub_g;

   ret=ioctl(fd,IFX_PMCU_INFO_GET,&param);
   close(fd);
#endif
   printf("Not supported!!\n");
   return;
}

/**
   The main function loops over the parameter list coming from the commandline
   and store them into internal variables. If all parameters are processed the 
   corresponding internal function is called.\n
   Current supported functions are:\n
   - pmcu_help()
   - set_state()
   - get_state()
   - set_loglevel()
   - set_enablePwrStateRequest() 
   - display_version()
   - display_pmcu_info()

   \return None
*/
int main (int argc, char** argv)
{
#if 0
   /* fetch the major number of the pmcu kernel driver from the proc file system */
   system("grep ifx_pmcu /proc/devices | cut -d ' ' -f 1 > pmcu_major_no.txt");
   fPtr = fopen("pmcu_major_no.txt","r");
   if(fPtr == NULL){
      perror("no ifx_pmcu major number could be find!");
      exit(-1);
   }
   fgets(major, 4, fPtr); /* get the 3 digit major number from file  */
   mknod("/dev/ifx_pmcu", (S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH), atoi(major)<<8);
#endif

   while(1) {
      int option_index=0;
      static struct option long_options[] ={
         {"help",0,0,0},
         {"set",1,0,0},
         {"module",1,0,0},
         {"sub",1,0,0},
         {"get",0,0,0},
         {"event",0,0,0},
         {"version",0,0,0},
         {"psinfo",0,0,0},
         {"log",1,0,0},
         {"req",1,0,0},
         {"fea",1,0,0}
      };
      c_opt_g = getopt_long(argc,argv, "hvm:n:s:l:r:f:ged", long_options, &option_index);

      //printf("c=%d option_index=%d\n",c,option_index);
      if(c_opt_g==-1) {
         if(input_flag_g==0) {
            printf("pmcu:please specify parameters\n");
            func=&pmcu_help;
         }
         if(func)
            (*func)();
         else {
ERROR:
            pmcu_help();
         }
         break;
      }
      input_flag_g=1;
      switch (c_opt_g) {
      case 0:
         if(option_index==0) {
            func=&pmcu_help;
            break;
         }
         if(option_index==1) {
            if(strcmp(optarg,"0")==0) {
               state_g=IFX_PMCU_STATE_D0;
            } else if(strcmp(optarg,"1")==0) {
               state_g=IFX_PMCU_STATE_D1;
            } else if(strcmp(optarg,"2")==0) {
               state_g=IFX_PMCU_STATE_D2;
            } else if(strcmp(optarg,"3")==0) {
               state_g=IFX_PMCU_STATE_D3;
            } else {
               printf("No such state!\n");
               exit(0);
            }
            func=&set_state;
            break;
         }
         if(option_index==2)
         {
            if(!optarg)
               goto ERROR;
            if(strcmp(optarg,"cpu")==0) {
               module_g=IFX_PMCU_MODULE_CPU;
            } else if(strcmp(optarg,"eth")==0) {
               module_g=IFX_PMCU_MODULE_ETH;
            } else if(strcmp(optarg,"usb")==0) {
               module_g=IFX_PMCU_MODULE_USB;
            } else if(strcmp(optarg,"dsl")==0) {
               module_g=IFX_PMCU_MODULE_DSL;
            } else if(strcmp(optarg,"wlan")==0) {
               module_g=IFX_PMCU_MODULE_WLAN;
            } else if(strcmp(optarg,"dect")==0) {
               module_g=IFX_PMCU_MODULE_DECT;
            } else if(strcmp(optarg,"fxs")==0) {
               module_g=IFX_PMCU_MODULE_FXS;
            } else if(strcmp(optarg,"fxo")==0) {
               module_g=IFX_PMCU_MODULE_FXO;
            } else if(strcmp(optarg,"ve")==0) {
               module_g=IFX_PMCU_MODULE_VE;
            } else if(strcmp(optarg,"ppe")==0) {
               module_g=IFX_PMCU_MODULE_PPE;
            } else if(strcmp(optarg,"switch")==0) {
               module_g=IFX_PMCU_MODULE_SWITCH;
            } else if(strcmp(optarg,"uart")==0) {
               module_g=IFX_PMCU_MODULE_UART;
            } else if(strcmp(optarg,"spi")==0) {
               module_g=IFX_PMCU_MODULE_SPI;
            } else if(strcmp(optarg,"sdio")==0) {
               module_g=IFX_PMCU_MODULE_SDIO;
            } else if(strcmp(optarg,"pci")==0) {
               module_g=IFX_PMCU_MODULE_PCI;
            } else if(strcmp(optarg,"vlynq")==0) {
               module_g=IFX_PMCU_MODULE_VLYNQ;
            } else if(strcmp(optarg,"deu")==0) {
               module_g=IFX_PMCU_MODULE_DEU;
            } else if (strcmp(optarg,"cpu_ps")==0) {
               module_g=IFX_PMCU_MODULE_CPU_PS;
            } else if (strcmp(optarg,"gptc")==0) {
               module_g=IFX_PMCU_MODULE_GPTC;
            } else if (strcmp(optarg,"usif_uart")==0) {
               module_g=IFX_PMCU_MODULE_USIF_UART;
            } else if (strcmp(optarg,"usif_spi")==0) {
               module_g=IFX_PMCU_MODULE_USIF_SPI;
            } else if (strcmp(optarg,"pcie")==0) {
               module_g=IFX_PMCU_MODULE_PCIE;
            } else {
               printf("No such module!\n");
               exit(0);
            }
            break;
         }
         if(option_index==3) {
            if(!optarg)
               goto ERROR;
            sub_g=strtoul(optarg,NULL,0);
            break;
         }
         if(option_index==4) {
            func=&get_state;
            break;
         }
         if(option_index==5) {
            func=&get_info;
            break;
         }
         if(option_index==6) {
            func=&display_version;
            break;
         }
         if(option_index==7) {
            func=&display_pmcu_info;
            break;
         }
         if(option_index==8) {
            if(strcmp(optarg,"0")==0) {
               log_level_g=0;
            } else if(strcmp(optarg,"1")==0) {
               log_level_g=1;
            } else if(strcmp(optarg,"2")==0) {
               log_level_g=2;
            } else if(strcmp(optarg,"3")==0) {
               log_level_g=3;
            } else {
               printf("No such log level!\n");
               exit(0);
            }
            func=&set_logLevel;
            break;
         }
         if(option_index==9) {
            if(strcmp(optarg,"0")==0) {
               req_ctrl_g=0;
            } else if(strcmp(optarg,"1")==0) {
               req_ctrl_g=1;
            } else {
               printf("only values 0=disable; 1=enable are allowed\n");
               exit(0);
            }
            func=&set_enablePwrStateRequest;
            break;
         }
         if(option_index==10) {
            if(strcmp(optarg,"ON")==0) {
               pwrFea_g=IFX_PMCU_PWR_STATE_ON;
            } else if(strcmp(optarg,"OFF")==0) {
               pwrFea_g=IFX_PMCU_PWR_STATE_OFF;
            } else {
               printf("No such power feature!\n");
               exit(0);
            }
            func=&set_pwrFeaState;
            break;
         }
      case 'h':
         func=&pmcu_help;
         break;
      case 's':
         if(strcmp(optarg,"0")==0) {
            state_g=IFX_PMCU_STATE_D0;
         } else if(strcmp(optarg,"1")==0) {
            state_g=IFX_PMCU_STATE_D1;
         } else if(strcmp(optarg,"2")==0) {
            state_g=IFX_PMCU_STATE_D2;
         } else if(strcmp(optarg,"3")==0) {
            state_g=IFX_PMCU_STATE_D3;
         } else {
            printf("No such state!\n");
            exit(0);
         }
         func=&set_state;
         break;
      case 'f':
         if((strcmp(optarg,"ON")==0) || (strcmp(optarg,"on")==0)) {
            pwrFea_g=IFX_PMCU_PWR_STATE_ON;
         } else if((strcmp(optarg,"OFF")==0) || (strcmp(optarg,"off")==0)) {
            pwrFea_g=IFX_PMCU_PWR_STATE_OFF;
         } else {
            printf("No such power feature!\n");
            exit(0);
         }
         func=&set_pwrFeaState;
         break;
      case 'g':
         func=&get_state;
         break;
      case 'l':
         if(strcmp(optarg,"0")==0) {
            log_level_g=0;
         } else if(strcmp(optarg,"1")==0) {
            log_level_g=1;
         } else if(strcmp(optarg,"2")==0) {
            log_level_g=2;
         } else if(strcmp(optarg,"3")==0) {
            log_level_g=3;
         } else {
            printf("No such log level!\n");
            exit(0);
         }
         func=&set_logLevel;
         break;
      case 'm':
         if(strcmp(optarg,"cpu")==0) {
            module_g=IFX_PMCU_MODULE_CPU;
         } else if(strcmp(optarg,"eth")==0) {
            module_g=IFX_PMCU_MODULE_ETH;
         } else if(strcmp(optarg,"usb")==0) {
            module_g=IFX_PMCU_MODULE_USB;
         } else if(strcmp(optarg,"dsl")==0) {
            module_g=IFX_PMCU_MODULE_DSL;
         } else if(strcmp(optarg,"wlan")==0) {
            module_g=IFX_PMCU_MODULE_WLAN;
         } else if(strcmp(optarg,"dect")==0) {
            module_g=IFX_PMCU_MODULE_DECT;
         } else if(strcmp(optarg,"fxs")==0) {
            module_g=IFX_PMCU_MODULE_FXS;
         } else if(strcmp(optarg,"fxo")==0) {
            module_g=IFX_PMCU_MODULE_FXO;
         } else if(strcmp(optarg,"ve")==0) {
            module_g=IFX_PMCU_MODULE_VE;
         } else if(strcmp(optarg,"ppe")==0) {
            module_g=IFX_PMCU_MODULE_PPE;
         } else if(strcmp(optarg,"switch")==0) {
            module_g=IFX_PMCU_MODULE_SWITCH;
         } else if(strcmp(optarg,"uart")==0) {
            module_g=IFX_PMCU_MODULE_UART;
         } else if(strcmp(optarg,"spi")==0) {
            module_g=IFX_PMCU_MODULE_SPI;
         } else if(strcmp(optarg,"sdio")==0) {
            module_g=IFX_PMCU_MODULE_SDIO;
         } else if(strcmp(optarg,"pci")==0) {
            module_g=IFX_PMCU_MODULE_PCI;
         } else if(strcmp(optarg,"vlynq")==0) {
            module_g=IFX_PMCU_MODULE_VLYNQ;
         } else if(strcmp(optarg,"deu")==0) {
            module_g=IFX_PMCU_MODULE_DEU;
         } else if (strcmp(optarg,"cpu_ps")==0) {
            module_g=IFX_PMCU_MODULE_CPU_PS;
         } else if (strcmp(optarg,"gptc")==0) {
            module_g=IFX_PMCU_MODULE_GPTC;
         } else if (strcmp(optarg,"usif_uart")==0) {
            module_g=IFX_PMCU_MODULE_USIF_UART;
         } else if (strcmp(optarg,"usif_spi")==0) {
            module_g=IFX_PMCU_MODULE_USIF_SPI;
         } else if (strcmp(optarg,"pcie")==0) {
            module_g=IFX_PMCU_MODULE_PCIE;
         } else {
            printf("No such module!\n");
            exit(0);
         }
         break;
      case 'n':
         if(!optarg)
            goto ERROR;
         sub_g=strtoul(optarg,NULL,0);
         break;
      case 'e':
         func=&get_info;
         break;
      case 'v':
         func=&display_version;
         break;
      case 'd':
         func=&display_pmcu_info;
         break;
      case 'r':
         if(strcmp(optarg,"0")==0) {
            req_ctrl_g=0;
         } else if(strcmp(optarg,"1")==0) {
            req_ctrl_g=1;
         } else {
            printf("only values 0=disable; 1=enable are allowed\n");
            exit(0);
         }
         func=&set_enablePwrStateRequest;
         break;
      default:
         break;
      }
   }
   return 0;
}
/*@}*/ /* LQ_PM */
