/******************************************************************************

                               Copyright (c) 2010
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/
#ifndef _COC_PM_H_
#define _COC_PM_H_


/*=============================================================================*/
/* PMCU IOCTL DEFINES                                                          */
/*=============================================================================*/
/** Set a given module into a specific power state

   \param IFX_PMCU_MODULE_STATE_t* Pointer to a
    \ref IFX_PMCU_MODULE_STATE_t structure.

   \return Returns value as follows:
     - IFX_SUCCESS: if successful
     - IFX_ERROR: in case of an error

   \code
   IFX_PMCU_MODULE_STATE_t param;
   param.pmcuModule=IFX_PMCU_MODULE_CPU;
   param.pmcuModuleNr=0;
   param.pmcuState=IFX_PMCU_STATE_D0;
   ioctl(fd, IFX_PMCU_STATE_REQ, &param);
   \endcode
*/
#define IFX_PMCU_STATE_REQ  0


/** Get the current power state from a given module
   \param IFX_PMCU_MODULE_STATE_t* Pointer to a
    \ref IFX_PMCU_MODULE_STATE_t structure.

   \return Returns value as follows:
     - IFX_SUCCESS: if successful
     - IFX_ERROR: in case of an error

   \code
   IFX_PMCU_MODULE_STATE_t param;
   param.pmcuModule=IFX_PMCU_MODULE_CPU;
   param.pmcuModuleNr=0;
   param.pmcuState=IFX_PMCU_STATE_INVALID;
   ioctl(fd, IFX_PMCU_STATE_GET, &param);
   \endcode
*/
#define IFX_PMCU_STATE_GET  1


/** Change the log level for the PMCU module
   \param integer value which specifies the log level
          - <0 = quiet
          -  0 = only ERRORS,WARNINGS (default)
          -  1 = + INFO
          -  2 = all

   \return Returns value as follows:
     - IFX_SUCCESS: if successful
     - IFX_ERROR: in case of an error

   \code
   int loglevel = 2;
   ioctl(fd, IFX_PMCU_LOG_LEVEL, &loglevel);
   \endcode
*/
#define IFX_PMCU_LOG_LEVEL  2


/** Control the acceptance of the powerstate request for the
    PMCU module. To have a central possibility for a global
    enable/disable of the powerstate requests coming from other
    modules, this IOCTL is introduced.
   \param integer value to enable/disable the powerstate 
          request inside the PMCU driver.
           - 0 =   All powerstate requests received by the PMCU
                   will be rejected. Default
           - 1  =  Powerstate requests are accepted by the PMCU

   \return Returns value as follows:
     - IFX_SUCCESS: if successful
     - IFX_ERROR: in case of an error

   \code
   enable the powerstate request acceptance
   int reqCtrl = 1; 
   ioctl(fd, IFX_PMCU_REQ_CTRL, &reqCtrl); 
   \endcode
*/
#define IFX_PMCU_REQ_CTRL  3

/** Switch the available power management features of the system
	on and off. 
	
   \param IFX_PMCU_MODULE_PWR_FEATURE_STATE_t* Pointer to struct
    \ref IFX_PMCU_MODULE_PWR_FEATURE_STATE_t structure.

   \return Returns value as follows:
     - IFX_SUCCESS: if successful
     - IFX_ERROR: in case of an error

   \code
   IFX_PMCU_MODULE_PWR_FEATURE_STATE_t param;
   param.pmcuModule=IFX_PMCU_MODULE_CPU;
   param.pmcuModuleNr=0;
   param.pmcuPwrFeatureState=IFX_PMCU_PWR_STATE_OFF;
   ioctl(fd, IFX_PMCU_PWR_FEATURE_SWITCH, &param); 
   \endcode
*/
#define IFX_PMCU_PWR_FEATURE_SWITCH  4

/*=============================================================================*/
/* PMCU ENUMERATION                                                                                                                */
/*=============================================================================*/
/** IFX_PMCU_MODULE_t
   Definition of the modules identifier
 */
typedef enum
{
   IFX_PMCU_MODULE_PMCU       = 0,
   IFX_PMCU_MODULE_CPU        = 1,
   IFX_PMCU_MODULE_ETH        = 2,
   IFX_PMCU_MODULE_USB        = 3,
   IFX_PMCU_MODULE_DSL        = 4,
   IFX_PMCU_MODULE_WLAN       = 5,
   IFX_PMCU_MODULE_DECT       = 6,
   IFX_PMCU_MODULE_FXS        = 7,
   IFX_PMCU_MODULE_FXO        = 8,
   IFX_PMCU_MODULE_VE         = 9,
   IFX_PMCU_MODULE_PPE        = 10,
   IFX_PMCU_MODULE_SWITCH     = 11,
   IFX_PMCU_MODULE_UART       = 12,
   IFX_PMCU_MODULE_SPI        = 13,
   IFX_PMCU_MODULE_SDIO       = 14,
   IFX_PMCU_MODULE_PCI        = 15,
   IFX_PMCU_MODULE_VLYNQ      = 16,
   IFX_PMCU_MODULE_DEU        = 17,
   IFX_PMCU_MODULE_CPU_PS     = 18,
   IFX_PMCU_MODULE_GPTC       = 19,
   IFX_PMCU_MODULE_USIF_UART  = 20,    /* VR9 usage */
   IFX_PMCU_MODULE_USIF_SPI   = 21,    /* VR9 usage */
   IFX_PMCU_MODULE_PCIE       = 22,    /* VR9 usage */
   IFX_PMCU_MODULE_ID_MAX     = 23,
} IFX_PMCU_MODULE_t;


/** IFX_PMCU_STATE_t
      Definition of power management state
*/
typedef enum
{
   /* ATTENTION:  Keep in mind that this list correspond to the ifx_pmcu_stateMap[][] array. 
                  If you have to change this list for any reason, ifx_pmcu_stateMap[][] must be adapted too. */

   /** State Invalid. */
   IFX_PMCU_STATE_INVALID = 0,
   /** State D0. Fully on */
   IFX_PMCU_STATE_D0 = 1,
   /** State D1. Device dependent */
   IFX_PMCU_STATE_D1 = 2,
   /** State D2. Device dependent */
   IFX_PMCU_STATE_D2 = 3,
   /** State D3. Off */
   IFX_PMCU_STATE_D3 = 4,
   /** don't care state */
   IFX_PMCU_STATE_D0D3 = 5,
} IFX_PMCU_STATE_t;

typedef enum
{
   /** State Invalid. */
   IFX_PMCU_PWR_STATE_INVALID = 0,
   /** Features State on */
   IFX_PMCU_PWR_STATE_ON = 1,
   /** Features State off */
   IFX_PMCU_PWR_STATE_OFF = 2,
} IFX_PMCU_PWR_STATE_ENA_t;

/*=============================================================================*/
/* PMCU STRUCTURES                                                             */
/*=============================================================================*/
/** IFX_PMCU_MODULE_STATE_t
   Structure hold the module-ID, the moduleSub-ID and the PowerState of one module.
*/
typedef struct
{
   /** Module identifier */
   IFX_PMCU_MODULE_t pmcuModule;
   /** instance identification of a Module; values 0,1,2,..... (0=first instance) */
   unsigned char     pmcuModuleNr;
   /** Module PowerState */
   IFX_PMCU_STATE_t  pmcuState;
} IFX_PMCU_MODULE_STATE_t;

/** IFX_PMCU_MODULE_PWR_FEATURE_STATE_t
   Structure hold the module-ID and the PowerFeature_State to be set for a dedicated module.
*/
typedef struct
{
   /** Module identifier */
   IFX_PMCU_MODULE_t pmcuModule;
   /** instance identification of a Module; values 0,1,2,..... (0=first instance) */
   unsigned char     pmcuModuleNr;
   /** Module PowerFeatureState */
   IFX_PMCU_PWR_STATE_ENA_t  pmcuPwrFeatureState;
} IFX_PMCU_MODULE_PWR_FEATURE_STATE_t;

#endif   /* _COC_PM_H_ */
